@@ -34,6 +34,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-url.h>
 #include <string.h>
+#include "yelp-error.h"
 #include "yelp-html.h"
 #include "yelp-util.h"
 #include "yelp-section.h"
@@ -62,57 +63,60 @@ static void        window_class_init	          (YelpWindowClass   *klass);
 static void        window_populate                (YelpWindow        *window);
 
 static gboolean    window_handle_uri              (YelpWindow        *window,
-						    YelpURI          *uri);
+						   YelpURI           *uri);
 static void        window_uri_selected_cb         (gpointer           view,
-						    YelpURI          *uri,
-						    gboolean          handled,
-						    YelpWindow       *window);
+						   YelpURI           *uri,
+						   gboolean           handled,
+						   YelpWindow        *window);
+static void        window_title_changed_cb        (gpointer           view,
+						   const gchar       *title,
+						   YelpWindow        *window);
 static void        window_toggle_history_back     (YelpHistory       *history,
-						    gboolean          sensitive,
-						    YelpWindow       *window);
-
+						   gboolean           sensitive,
+						   YelpWindow        *window);
+ 
 static void        window_toggle_history_forward  (YelpHistory       *history,
 						   gboolean           sensitive,
 						   YelpWindow        *window);
 
 static void        window_history_action          (YelpWindow        *window,
 						   YelpHistoryAction  action);
-static void        window_back_button_clicked     (GtkWidget        *button,
-						   YelpWindow       *window);
-static void        window_forward_button_clicked  (GtkWidget        *button,
-						   YelpWindow       *window);
-static void        window_home_button_clicked     (GtkWidget        *button,
-						   YelpWindow       *window);
-static void        window_index_button_clicked    (GtkWidget        *button,
-						   YelpWindow       *window);
-static void        window_new_window_cb           (gpointer          data,
-						   guint             section,
-						   GtkWidget        *widget);
-static void        window_close_window_cb         (gpointer          data,
-						   guint             section,
-						   GtkWidget        *widget);
-static void        window_history_go_cb           (gpointer          data,
-						   guint             section,
-						   GtkWidget        *widget);
-static void        window_go_home_cb              (gpointer          data,
-						   guint             section,
-						   GtkWidget        *widget);
-static void        window_go_index_cb             (gpointer          data,
-						   guint             section,
-						   GtkWidget        *widget);
-static void        window_about_cb                (gpointer          data,
-						   guint             section,
-						   GtkWidget        *widget);
-static GtkWidget * window_create_toolbar          (YelpWindow       *window);
+static void        window_back_button_clicked     (GtkWidget         *button,
+						   YelpWindow        *window);
+static void        window_forward_button_clicked  (GtkWidget         *button,
+						   YelpWindow        *window);
+static void        window_home_button_clicked     (GtkWidget         *button,
+						   YelpWindow        *window);
+static void        window_index_button_clicked    (GtkWidget         *button,
+						   YelpWindow        *window);
+static void        window_new_window_cb           (gpointer           data,
+						   guint              section,
+						   GtkWidget         *widget);
+static void        window_close_window_cb         (gpointer           data,
+						   guint              section,
+						   GtkWidget         *widget);
+static void        window_history_go_cb           (gpointer           data,
+						   guint              section,
+						   GtkWidget         *widget);
+static void        window_go_home_cb              (gpointer           data,
+						   guint              section,
+						   GtkWidget         *widget);
+static void        window_go_index_cb             (gpointer           data,
+						   guint              section,
+						   GtkWidget         *widget);
+static void        window_about_cb                (gpointer           data,
+						   guint              section,
+						   GtkWidget         *widget);
+static GtkWidget * window_create_toolbar          (YelpWindow        *window);
 
-static void        window_toolbar_style_changed_cb (GConfClient     *client,
-						    guint            cnxn_id,
-						    GConfEntry      *entry,
-						    gpointer         data);
+static void        window_toolbar_style_changed_cb (GConfClient      *client,
+						    guint             cnxn_id,
+						    GConfEntry       *entry,
+						    gpointer          data);
 
 #if 0
-static void        window_remove_notify_cb        (GtkObject        *obj, 
-						   gpointer          data);
+static void        window_remove_notify_cb        (GtkObject         *obj, 
+						   gpointer           data);
 #endif
 
 enum {
@@ -327,7 +331,16 @@ window_handle_uri (YelpWindow *window, YelpURI *uri)
 
 	d(g_print ("Handling URL: %s\n", yelp_uri_to_string (uri)));
 
-	if (yelp_uri_get_type (uri) == YELP_URI_TYPE_TOC) {
+	if (!yelp_uri_exists (uri)) {
+		gchar *str_uri = yelp_uri_to_string (uri);
+		
+		g_set_error (&error,
+			     YELP_ERROR,
+			     YELP_ERROR_URI_NOT_EXIST,
+			     _("The document '%s' does not exist"), str_uri);
+		g_free (str_uri);
+	}
+	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_TOC) {
 		d(g_print ("[TOC]\n"));
 		
 		yelp_view_toc_open_uri (YELP_VIEW_TOC (priv->toc_view), uri);
@@ -335,10 +348,14 @@ window_handle_uri (YelpWindow *window, YelpURI *uri)
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 					       PAGE_TOC_VIEW);
 		handled = TRUE;
-	} 
+	}
 	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_INDEX) {
+		d(g_print ("[INDEX]\n"));
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 					       PAGE_INDEX_VIEW);
+		yelp_view_index_show_uri (YELP_VIEW_INDEX (priv->index_view),
+					  uri, 
+					  &error);
 		handled = TRUE;
 	}
 	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_MAN ||
@@ -402,6 +419,21 @@ window_uri_selected_cb (gpointer    view,
 	yelp_uri_unref (uri);
 }
 
+static void
+window_title_changed_cb (gpointer view, const gchar *title, YelpWindow *window)
+{
+	gchar *new_title;
+	
+	g_return_if_fail (title != NULL);
+	g_return_if_fail (YELP_IS_WINDOW (window));
+	
+	new_title = g_strconcat (_("Help Browser"), ":", title, NULL);
+
+	gtk_window_set_title (GTK_WINDOW (window), new_title);
+
+	g_free (new_title);
+}
+
 static void
 window_toggle_history_back (YelpHistory *history,
 			    gboolean     sensitive,
@@ -574,6 +606,7 @@ window_about_cb (gpointer data, guint section, GtkWidget *widget)
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 NULL);
 
+	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (data));
 	gtk_widget_show (about);
 }
 
@@ -735,10 +768,18 @@ yelp_window_new (GNode *doc_tree, GList *index)
 			  G_CALLBACK (window_uri_selected_cb),
 			  window);
 
+	g_signal_connect (priv->toc_view, "title_changed",
+			  G_CALLBACK (window_title_changed_cb),
+			  window);
+
 	g_signal_connect (priv->content_view, "uri_selected",
 			  G_CALLBACK (window_uri_selected_cb),
 			  window);
 
+	g_signal_connect (priv->content_view, "title_changed",
+			  G_CALLBACK (window_title_changed_cb),
+			  window);
+
 	window_populate (window);
 
 	yelp_window_open_uri (window, "toc:");
@@ -759,9 +800,7 @@ yelp_window_open_uri (YelpWindow  *window,
 
 	uri = yelp_uri_new (str_uri);
 
-	if (yelp_uri_exists (uri)) {
-		window_handle_uri (window, uri);
-	}
+	window_handle_uri (window, uri);
 	
 	yelp_uri_unref (uri);
 }
